package java.security;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.List;
import java.security.cert.CertPath;
import java.util.Date;
import java.io.Serializable;

public final class Timestamp implements Serializable
{
    private static final long serialVersionUID = -5502683707821851294L;
    private Date timestamp;
    private CertPath signerCertPath;
    private transient int myhash;
    
    public Timestamp(Date date, CertPath signerCertPath) {
        this.myhash = -1;
        if (date == null || signerCertPath == null) {
            throw new NullPointerException();
        }
        this.timestamp = new Date(date.getTime());
        this.signerCertPath = signerCertPath;
        return;
    }
    
    public Date getTimestamp() {
        return new Date(this.timestamp.getTime());
    }
    
    public CertPath getSignerCertPath() {
        return this.signerCertPath;
    }
    
    @Override
    public int hashCode() {
        if (this.myhash == -1) {
            this.myhash = this.timestamp.hashCode() + this.signerCertPath.hashCode();
        }
        return this.myhash;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == null || !(o instanceof Timestamp)) {
            return false;
        }
        Timestamp timestamp = (Timestamp)o;
        return this == timestamp || (this.timestamp.equals(timestamp.getTimestamp()) && this.signerCertPath.equals(timestamp.getSignerCertPath()));
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("(");
        sb.append("timestamp: " + this.timestamp);
        List<Object> certificates = this.signerCertPath.getCertificates();
        if (!certificates.isEmpty()) {
            sb.append("TSA: " + certificates.get(0));
        }
        else {
            sb.append("TSA: <empty>");
        }
        sb.append(")");
        return sb.toString();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.myhash = -1;
        this.timestamp = new Date(this.timestamp.getTime());
    }
}
